from django.db import models


class AbstractCyberSourceTransaction(models.Model):
    """
    Stores credit card transaction receipts made with CyberSource.
    """
    id = models.BigAutoField(primary_key=True)
    uuid = models.CharField(max_length=32)
    created = models.DateTimeField(auto_now_add=True)
    return_from_cybersource = models.DateTimeField(null=True, blank=True)

    class Meta:
        abstract = True



import datetime
from base64 import b64encode
from hashlib import sha256
import hmac

from django.conf import settings


def create_sha256_signature(key, message):
    """
    Signs an HMAC SHA-256 signature to a message with Base 64
    encoding. This is required by CyberSource.
    """
    digest = hmac.new(
        key.encode(),
        msg=message.encode(),
        digestmod=sha256,
    ).digest()
    return b64encode(digest).decode()


def sign_fields_to_context(fields, context):
    """
    Builds the list of file names and data to sign, and created the
    signature required by CyberSource.
    """
    fields['signed_date_time'] = str(datetime.datetime.utcnow().isoformat(timespec='seconds')) + 'Z'
    signed_field_names = []
    data_to_sign = []
    for key, value in fields.items():
        signed_field_names.append(key)

    signed_field_names.append('unsigned_field_names')
    fields['unsigned_field_names'] = ''
    signed_field_names.append('signed_field_names')
    fields['signed_field_names'] = ','.join(signed_field_names)

    for key, value in fields.items():
        data_to_sign.append(f'{key}={value}')

    context['fields'] = fields
    context['signature'] = create_sha256_signature(
        settings.CYBERSOURCE_SECRET_KEY,
        ','.join(data_to_sign),
    )
    context['url'] = settings.CYBERSOURCE_URL

    return context



from setuptools import setup, find_packages

with open('README.md') as f:
    long_description = f.read()

setup(
    name='django-cybersource-hosted-checkout',
    version="0.0.5",
    description='This package provides utilities for using CyberSource Secure Acceptance Hosted Checkout.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Tim Allen',
    author_email='tallen@wharton.upenn.edu',
    url='https://github.com/wharton/django-cybersource-hosted-checkout',
    include_package_data=True,
    packages=find_packages(),
    zip_safe=False,
    install_requires=[
        'Django',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Framework :: Django',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
